@@ -1,6 +1,5 @@
 package MooseX::NonMoose::Meta::Role::Class;
-our $VERSION = '0.06';
-
+our $VERSION = '0.07';
 
 use Moose::Role;
 use List::MoreUtils qw(any);
@@ -11,22 +10,16 @@ MooseX::NonMoose::Meta::Role::Class - metaclass trait for L<MooseX::NonMoose>
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
   package Foo;
-our $VERSION = '0.06';
-
-
   use Moose -traits => 'MooseX::NonMoose::Meta::Role::Class';
 
   # or
 
   package My::Moose;
-our $VERSION = '0.06';
-
-
   use Moose ();
   use Moose::Exporter;
 
@@ -56,14 +49,15 @@ has has_nonmoose_constructor => (
     default => 0,
 );
 
-around _immutable_options => sub {
-    my $orig = shift;
+has has_nonmoose_destructor => (
+    is  => 'rw',
+    isa => 'Bool',
+    default => 0,
+);
+
+sub _determine_constructor_options {
     my $self = shift;
-
-    my @options = $self->$orig(@_);
-
-    # do nothing if extends was never called
-    return @options if !$self->has_nonmoose_constructor;
+    my @options = @_;
 
     # if we're using just the metaclass trait, but not the constructor trait,
     # then suppress the warning about not inlining a constructor
@@ -89,19 +83,33 @@ around _immutable_options => sub {
     # the warning message), since this is the expected usage, and shouldn't
     # cause a warning
     return (replace_constructor => 1, @options);
-};
+}
 
-around superclasses => sub {
+sub _determine_destructor_options {
+    my $self = shift;
+    my @options = @_;
+
+    return (@options, inline_destructor => 0);
+}
+
+around _immutable_options => sub {
     my $orig = shift;
     my $self = shift;
 
-    return $self->$orig unless @_;
+    my @options = $self->$orig(@_);
 
-    my @superclasses = @_;
-    push @superclasses, 'Moose::Object'
-        unless grep { $_->isa('Moose::Object') } @superclasses;
+    # do nothing if extends was never called
+    return @options if !$self->has_nonmoose_constructor
+                    && !$self->has_nonmoose_destructor;
 
-    my @ret = $self->$orig(@superclasses);
+    @options = $self->_determine_constructor_options(@options);
+    @options = $self->_determine_destructor_options(@options);
+
+    return @options;
+};
+
+sub _check_superclass_constructor {
+    my $self = shift;
 
     # if the current class defined a custom new method (since subs happen at
     # BEGIN time), don't try to override it
@@ -114,7 +122,7 @@ around superclasses => sub {
     my $super_new = $self->find_next_method_by_name('new');
 
     # if we're trying to extend a (non-immutable) moose class, just do nothing
-    return @ret if $super_new->package_name eq 'Moose::Object';
+    return if $super_new->package_name eq 'Moose::Object';
 
     if ($super_new->associated_metaclass->can('constructor_class')) {
         my $constructor_class_meta = Class::MOP::Class->initialize(
@@ -123,13 +131,13 @@ around superclasses => sub {
 
         # if the constructor we're inheriting is already one of ours, there's
         # no reason to install a new one
-        return @ret if $constructor_class_meta->can('does_role')
-                    && $constructor_class_meta->does_role('MooseX::NonMoose::Meta::Role::Constructor');
+        return if $constructor_class_meta->can('does_role')
+               && $constructor_class_meta->does_role('MooseX::NonMoose::Meta::Role::Constructor');
 
         # if the constructor we're inheriting is an inlined version of the
         # default moose constructor, don't do anything either
-        return @ret if any { $_->isa($constructor_class_meta->name) }
-                           $super_new->associated_metaclass->_inlined_methods;
+        return if any { $_->isa($constructor_class_meta->name) }
+                      $super_new->associated_metaclass->_inlined_methods;
     }
 
     $self->add_method(new => sub {
@@ -148,6 +156,70 @@ around superclasses => sub {
         return $self;
     });
     $self->has_nonmoose_constructor(1);
+}
+
+sub _check_superclass_destructor {
+    my $self = shift;
+
+    # if the current class defined a custom DESTROY method (since subs happen
+    # at BEGIN time), don't try to override it
+    return if $self->has_method('DESTROY');
+
+    # we need to get the non-moose destructor from the superclass
+    # of the class where this method actually exists, regardless of what class
+    # we're calling it on
+    my $super_DESTROY = $self->find_next_method_by_name('DESTROY');
+
+    # if we're trying to extend a (non-immutable) moose class, just do nothing
+    return if $super_DESTROY->package_name eq 'Moose::Object';
+
+    if ($super_DESTROY->associated_metaclass->can('destructor_class')
+     && $super_DESTROY->associated_metaclass->destructor_class) {
+        my $destructor_class_meta = Class::MOP::Class->initialize(
+            $super_DESTROY->associated_metaclass->destructor_class
+        );
+
+        # if the destructor we're inheriting is an inlined version of the
+        # default moose destructor, don't do anything
+        return if any { $_->isa($destructor_class_meta->name) }
+                      $super_DESTROY->associated_metaclass->_inlined_methods;
+    }
+
+    $self->add_method(DESTROY => sub {
+        my $self = shift;
+
+        local $?;
+
+        Try::Tiny::try {
+            $super_DESTROY->execute($self);
+            $self->DEMOLISHALL(Devel::GlobalDestruction::in_global_destruction);
+        }
+        Try::Tiny::catch {
+            # Without this, Perl will warn "\t(in cleanup)$@" because of some
+            # bizarre fucked-up logic deep in the internals.
+            no warnings 'misc';
+            die $_;
+        };
+
+        return;
+    });
+    $self->has_nonmoose_destructor(1);
+}
+
+around superclasses => sub {
+    my $orig = shift;
+    my $self = shift;
+
+    return $self->$orig unless @_;
+
+    my @superclasses = @_;
+    push @superclasses, 'Moose::Object'
+        unless grep { $_->isa('Moose::Object') } @superclasses;
+
+    my @ret = $self->$orig(@superclasses);
+
+    $self->_check_superclass_constructor;
+    $self->_check_superclass_destructor;
 
     return @ret;
 };
